﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using EikonDesktopDataAPI;
using Dex2;

namespace CSharpEikonnect
{
    public partial class Form1 : Form,_IEikonDesktopDataAPIEvents,_IRDataEvents
    {

        EikonDesktopDataAPI.EikonDesktopDataAPI MyEikonDesktopDataAPI = null;

        Dex2.Dex2Mgr MyDex2Mgr = null;
        Dex2.IDex2Mgr2 MyDex2MgrADC = null;
        int MyDex2Cookie = 0;
        Dex2.RData MyRData = null;

        public Form1()
        {
            //This call is required by the designer.
            InitializeComponent();

            // Add any initialization after the InitializeComponent() call.
            UpdateUserFormAccordingToConnectionStatus(EikonDesktopDataAPI.EEikonStatus.Offline);
            UpdateDex2FormAccordingToDex2Status();
        }

    //================================================================================================
    // Step 1 - Instantiate EikonDesktopDataAPILib.EikonDesktopDataAPI and connect to its outgoing interface =
    //================================================================================================

        private void ConnectToEikon()
        {

            //Create the Eikon Desktop object
            MyEikonDesktopDataAPI = new EikonDesktopDataAPI.EikonDesktopDataAPI();
            
            // Initialize the Eikon Desktop object to connect to Eikon
            if (MyEikonDesktopDataAPI != null)
            {

                EEikonDataAPIInitializeResult l_result = MyEikonDesktopDataAPI.Initialize();
                if (l_result != EEikonDataAPIInitializeResult.Succeed)
                {
                    DisconnectFromEikon();
                    if (l_result == EEikonDataAPIInitializeResult.Error_Reinitialize)
                    {
                        DataRetrievedTxt.Text = "Reinitialization of EikonDesktopDataAPI is forbidden";
                    }
                    else if (l_result == EEikonDataAPIInitializeResult.Error_InitializeFail)
                    {
                        DataRetrievedTxt.Text = "Initialization of EikonDesktopDataAPI Fail";
                    }
                    UpdateUserFormAccordingToConnectionStatus(EEikonStatus.Disconnected);
                    return;
                }
            }
            MyEikonDesktopDataAPI.OnStatusChanged += OnStatusChanged;
        }


        private void DisconnectFromEikon()
        {
            // Release the Dex2 manager is necessary
            ReleaseDex2Mgr();

            // Release Eikonnect
            MyEikonDesktopDataAPI = null;
        }

        public void OnStatusChanged(EEikonStatus EStatus)
        {
            //Update the use form according to the status of the connection to Eikon
            UpdateUserFormAccordingToConnectionStatus(EStatus);
        }

        //======================================
        // Step 2 - Instantiate a Dex2 manager =
        //======================================


        private void CreateDex2Mgr()
        {
            // Create the Dex2 manager
            MyDex2Mgr = MyEikonDesktopDataAPI.CreateDex2Mgr();
            MyDex2MgrADC = (Dex2.IDex2Mgr2) MyDex2Mgr;

            // Initialize the Dex2 manager and retrieve a session cookie
            if (MyDex2Mgr != null)
                MyDex2Cookie = MyDex2MgrADC.Initialize(DEX2_MetadataCaller.DE_MC_ADC_POWERLINK);
        }

        private void ReleaseDex2Mgr()
        {
            //Release the RData if necessary
            MyRData = null;

            // Release the Dex2 manager
            MyDex2Mgr = null;
        }

        //======================================
        // Step #3 : Create a RData to request =
        //======================================


        private void CreateRData(string InstrumentIDList,
                                string FieldList,
                                string RequestParam,
                                string DisplayParam)
        {
            // Create a RData
            MyRData = MyDex2Mgr.CreateRData(MyDex2Cookie);
            MyRData.OnUpdate += OnUpdate;
            // Initialize the RData to request
            if (MyRData != null)
            {
                MyRData.InstrumentIDList = InstrumentIDList;
                MyRData.FieldList = FieldList;
                MyRData.RequestParam = RequestParam;
                MyRData.DisplayParam = DisplayParam;
                RequestDataBtn.Enabled = true;
            }
        }

        public void OnUpdate(DEX2_DataStatus DataStatus, object Error)
        {
            // Handle the Dex2 response for the RData requested
            UpdateUserFormAccordingToRDataResponse(DataStatus, Error);
        }

        //======================================================
        // Step #4 : Send the request to Dex2 and get the data =
        //======================================================
        
        private void RequestDex2Data()
        {
            //Send the RData request to Dex2
            if (MyRData != null)
                MyRData.Subscribe();
        }


        //==========================================================================================================================
        //==========================================================================================================================

        //----------------------------------------------------------------------


        private void UpdateDex2FormAccordingToDex2Status()
        {
            if (MyDex2Mgr != null)
            {
                CreateDex2ManagerBtn.Enabled = false;
                ReleaseDex2ManagerBtn.Enabled = true;
                RDataGroupBox.Enabled = true;
                if (InstrumentsListTxt.TextLength != 0 && FieldsListTxt.TextLength != 0)
                    CreateRDataBtn.Enabled = true;
                else
                    CreateRDataBtn.Enabled = false;
            }
            else
            {
                CreateDex2ManagerBtn.Enabled = true;
                ReleaseDex2ManagerBtn.Enabled = false;
                RDataGroupBox.Enabled = false;
                RequestGroupBox.Enabled = false;
            }

        }

        private void UpdateUserFormAccordingToConnectionStatus(EikonDesktopDataAPI.EEikonStatus EStatus)
        {
            switch (EStatus)
            {
                case EEikonStatus.Connected:
                    ConnectToEikonBtn.Enabled = false;
                    DisconnectFromEikonBtn.Enabled = true;
                    StatusPictureBox.Image = Properties.Resources.Green;
                    ToolStripStatusLabel1.Text = "Status : Connected to Eikon";
                    Dex2MgrGroupBox.Enabled = true;
                    RDataGroupBox.Enabled = true;
                    RequestGroupBox.Enabled = true;
                    break;

                case EikonDesktopDataAPI.EEikonStatus.Disconnected:
                    DisconnectFromEikon();
                    ConnectToEikonBtn.Enabled = true;
                    DisconnectFromEikonBtn.Enabled = false;
                    StatusPictureBox.Image = Properties.Resources.Red;
                    ToolStripStatusLabel1.Text = "Status : Disconnected from Eikon";
                    Dex2MgrGroupBox.Enabled = false;
                    RDataGroupBox.Enabled = false;
                    RequestGroupBox.Enabled = false;
                    break;

                case EikonDesktopDataAPI.EEikonStatus.LocalMode:
                    ReleaseDex2Mgr();
                    ConnectToEikonBtn.Enabled = true;
                    DisconnectFromEikonBtn.Enabled = false;
                    StatusPictureBox.Image = Properties.Resources.Orange;
                    ToolStripStatusLabel1.Text = "Status : Eikon local mode";
                    Dex2MgrGroupBox.Enabled = false;
                    RDataGroupBox.Enabled = false;
                    RequestGroupBox.Enabled = false;
                    break;

                case EikonDesktopDataAPI.EEikonStatus.Offline:
                    DisconnectFromEikon();
                    ConnectToEikonBtn.Enabled = true;
                    DisconnectFromEikonBtn.Enabled = false;
                    StatusPictureBox.Image = Properties.Resources.Red;
                    ToolStripStatusLabel1.Text = "Status : Offline";
                    Dex2MgrGroupBox.Enabled = false;
                    RDataGroupBox.Enabled = false;
                    RequestGroupBox.Enabled = false;
                    break;
            }

            UpdateDex2FormAccordingToDex2Status();
        }




        private void ConnectToEikonBtn_Click(object sender, EventArgs e)
        {
            ToolStripStatusLabel1.Text = "Status : Connecting to Eikon...";
            ConnectToEikonBtn.Enabled = false;
            DisconnectFromEikonBtn.Enabled = true;

            ConnectToEikon();
        }

        private void DisconnectFromEikonBtn_Click(object sender, EventArgs e)
        {
            ToolStripStatusLabel1.Text = "Status : Disconnecting from Eikon...";
            ConnectToEikonBtn.Enabled = true;
            DisconnectFromEikonBtn.Enabled = false;

            DisconnectFromEikon();

            UpdateUserFormAccordingToConnectionStatus(EEikonStatus.Offline);
        }

        private void CreateDex2ManagerBtn_Click(object sender, EventArgs e)
        {
            CreateDex2Mgr();
            UpdateDex2FormAccordingToDex2Status();
        }

        private void ReleaseDex2ManagerBtn_Click(object sender, EventArgs e)
        {
            ReleaseDex2Mgr();
            UpdateDex2FormAccordingToDex2Status();
        }

        private void InstrumentsListTxt_TextChanged(object sender, EventArgs e)
        {
            MyRData = null;
            RequestGroupBox.Enabled = false;
            if (MyDex2Mgr != null && InstrumentsListTxt.TextLength != 0 && FieldsListTxt.TextLength != 0)
                CreateRDataBtn.Enabled = true;
            else
            {
                CreateRDataBtn.Enabled = false;
                //RequestGroupBox.Enabled = false;
            }
        }

        private void FieldsListTxt_TextChanged(object sender, EventArgs e)
        {
            MyRData = null;
            RequestGroupBox.Enabled = false;
            if (MyDex2Mgr != null && InstrumentsListTxt.TextLength != 0 && FieldsListTxt.TextLength != 0)
                CreateRDataBtn.Enabled = true;
            else
            {
                CreateRDataBtn.Enabled = false;
                //RequestGroupBox.Enabled = false;
            }
        }

        private void RequestParametersTxt_TextChanged(object sender, EventArgs e)
        {
            RequestGroupBox.Enabled = false;
            MyRData = null;
        }

        private void DisplayParametersTxt_TextChanged(object sender, EventArgs e)
        {
            RequestGroupBox.Enabled = false;
            MyRData = null;
        }

        private void CreateRDataBtn_Click(object sender, EventArgs e)
        {
            CreateRData(InstrumentsListTxt.Text, FieldsListTxt.Text, RequestParametersTxt.Text, DisplayParametersTxt.Text);

            DataRetrievedTxt.Clear();

            if (MyRData != null)
                RequestGroupBox.Enabled = true;
            else
                RequestGroupBox.Enabled = false;
        }

        //----------------------------------------------------------------------
        private void UpdateUserFormAccordingToRDataResponse(Dex2.DEX2_DataStatus DataStatus, Object responseError)
        {
            DataRetrievedLbl.Text = "Data status: " + DataStatus.ToString();
            switch (DataStatus)
            {
                case DEX2_DataStatus.DE_DS_FULL:
                    long i = 0;
                    long j = 0;
                    Array l_response = MyRData.Data as Array;
                   
                    for( i =  l_response.GetLowerBound(0); i <=  l_response.GetUpperBound(0);++i)
                    {
                            long l_row;
                            l_row = i - l_response.GetLowerBound(0) + 1;
                            for( j = l_response.GetLowerBound(1); j <= l_response.GetUpperBound(1);++j)
                            {                              
                                // error handling could be done here
                                if (MyRData.Data[i, j] != null)
                                    DataRetrievedTxt.Text += " " + MyRData.Data[i, j].ToString();
                            }
                            DataRetrievedTxt.Text += Environment.NewLine;
                    }
                    break;

                case DEX2_DataStatus.DE_DS_NULL_EMPTY:
                    if (responseError != null)
                        DataRetrievedTxt.Text = "Error : " + responseError;
                    else
                        DataRetrievedTxt.Text = "Error : DE_DS_NULL_EMPTY";

                    break;
                case DEX2_DataStatus.DE_DS_NULL_ERROR:
                    if (responseError != null)
                        DataRetrievedTxt.Text = "Error : " + responseError;
                    else
                        DataRetrievedTxt.Text = "Error : DE_DS_NULL_ERROR";

                    break;
                case DEX2_DataStatus.DE_DS_NULL_TIMEOUT:
                    if (responseError != null)
                        DataRetrievedTxt.Text = "Error : " + responseError;
                    else
                        DataRetrievedTxt.Text = "Error : DE_DS_NULL_TIMEOUT";

                    break;
                case DEX2_DataStatus.DE_DS_PARTIAL:
                    if (responseError != null)
                        DataRetrievedTxt.Text = "Error : " + responseError;
                    else
                        DataRetrievedTxt.Text = "Error : DE_DS_PARTIAL";

                    break;
            }
            CreateRDataBtn.Enabled = true;
            RequestDataBtn.Enabled = true;
        }

        private void RequestDataBtn_Click(object sender, EventArgs e)
        {
            RequestDataBtn.Enabled = false;
            CreateRDataBtn.Enabled = false;
            DataRetrievedLbl.Text = "Retrieve Data...";

            RequestDex2Data();
        }

        private void ClearRetrievedDataBtn_Click(object sender, EventArgs e)
        {
            DataRetrievedTxt.Clear();
        }

        private void EgInstrumentsListBtn_Click(object sender, EventArgs e)
        {
            InstrumentsListTxt.Text = "MDT.N;KSS.N;NI.N";
        }

        private void EgFieldsListBtn_Click(object sender, EventArgs e)
        {
            FieldsListTxt.Text = "TR.Revenue;TR.EBITDA";
        }

        private void EgRequesParamsBtn_Click(object sender, EventArgs e)
        {
            RequestParametersTxt.Text = "Period=FY-5";
        }

        private void EgDisplayParamsBtn_Click(object sender, EventArgs e)
        {
            DisplayParametersTxt.Text = "RH:In CH:Fd";
        }
    }
}
